# $Id: didyoumean.py 3919 2010-08-31 06:51:38Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.5.0/src/delfttilburg/didyoumean.py $
# -*- coding: utf-8 -*-
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
from meresco.core import Observable

from cqlparser import parseString, CqlIdentityVisitor, CQLParseException

class DidYouMean(Observable):

    def __init__(self, fieldnameConversion=None):
        Observable.__init__(self)
        self._fieldnameConversion = fieldnameConversion if fieldnameConversion else lambda x:x

    def _suggestionsForTerms(self, terms):
        suggestions = []
        term = ''
        if len(terms) == 1:
            fieldname, term = terms[0]
            inclusive, suggestions = self.any.suggestionsFor(term, fieldname=self._fieldnameConversion(fieldname))
        else:
            for (fieldname, originalTerm) in terms:
                inclusive, termSuggestions = self.any.suggestionsFor(originalTerm, fieldname=self._fieldnameConversion(fieldname))
                if termSuggestions and not inclusive:
                    term = originalTerm
                    suggestions = termSuggestions
                    break

        return term, suggestions

    def suggestionsForAst(self, ast):
        """Return the suggestions for the first term in the query that is not
        present in the index.
        """
        #query = ''.join(char for char in query if char not in ['<', '>', '/', '=', '(', ')'])
        class DidYouMeanVisitor(CqlIdentityVisitor):
            def visitSEARCH_CLAUSE(mii, node):
                children = node.children()
                if len(children) == 1 and children[0].children()[0].name() == 'TERM':
                    #SEARCH_CLAUSE(SEARCH_TERM(TERM(<term>)))
                    term = children[0].children()[0].children()[0]
                    terms.append((None, term))
                elif len(children) == 3 and children[2].children()[0].name() == 'TERM':
                    #SEARCH_CLAUSE(INDEX(TERM(<index>)), RELATION(...), SEARCH_TERM(TERM(<term>)))
                    index = children[0].children()[0].children()[0]
                    term = children[2].children()[0].children()[0]
                    terms.append((index, term))
                return CqlIdentityVisitor.visitSEARCH_CLAUSE(mii, node)

        terms = []
        DidYouMeanVisitor(ast).visit()
        return self._suggestionsForTerms(terms)

